package services;

import entities.AnnouncePackage;
import enums.ExecArgumentsEnum;
import enums.PortConfigurationEnum;
import enums.ProjectNameEnum;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.concurrent.atomic.AtomicBoolean;

public class AnnouncerService extends Thread {
    private IpAddressRetrievalService ipService = IpAddressRetrievalService.getInstance();
    private LogService log = LogService.getInstance();
    private SerializerService serializerService = SerializerService.getInstance();
    private DatagramSocket datagramSocket;
    private Integer targetPort;
    private AtomicBoolean awaitingShutdown = new AtomicBoolean(false);

    private String announcementInstanceType;
    private String announcementIp;
    private int announcementPort;

    public AnnouncerService() {
        try {
            datagramSocket = new DatagramSocket();
        } catch (SocketException e) {
            e.printStackTrace();
        }
    }

    public AtomicBoolean getAwaitingShutdown() {
        return awaitingShutdown;
    }

    @Override
    public void run() {
        AnnouncePackage announcePackage = new AnnouncePackage()
                .setInstanceType(announcementInstanceType)
                .setIp(announcementIp)
                .setPort(announcementPort);

        String serializedAnouncementPackage = serializerService.serialize(announcePackage);
        byte[] announcementData = serializedAnouncementPackage.getBytes();
        InetAddress broadcastAddress = ipService.getBroadcastIpAddress();

        while (!awaitingShutdown.get()) {
            DatagramPacket datagramPacket = new DatagramPacket(
                    announcementData,
                    announcementData.length,
                    broadcastAddress,
                    targetPort);
            try {
                datagramSocket.send(datagramPacket);
                Thread.sleep(1000);
            } catch (IOException e) {
                log.log(this, "IOException");
                e.printStackTrace();
            } catch (InterruptedException e) {
                log.log(this, "Expected InterruptedException");
                break;
            }
        }
    }

    public AnnouncerService setTargetPort(Integer targetPort) {
        this.targetPort = targetPort;
        return this;
    }

    public AnnouncerService setAnnouncementInstanceType(String announcementInstanceType) {
        this.announcementInstanceType = announcementInstanceType;
        return this;
    }

    public AnnouncerService setAnnouncementIp(String announcementIp) {
        this.announcementIp = announcementIp;
        return this;
    }

    public AnnouncerService setAnnouncementPort(int announcementPort) {
        this.announcementPort = announcementPort;
        return this;
    }
}
