package services;

import containers.AbstractSynchronizedResource;
import entities.AnnouncePackage;
import enums.ProjectNameEnum;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

public class FinderService extends Thread {
    private String listenProject = ProjectNameEnum.PROJECT_NAME.getContent();
    private int listenPort;
    private DatagramSocket datagramSocket;
    private LogService log = LogService.getInstance();
    private AbstractSynchronizedResource<Boolean> awaitingTermination = new AbstractSynchronizedResource<>();
    private SerializerService serializerService = new SerializerService();
    private AbstractSynchronizedResource<InetAddress> addressContainer;

    @Override
    public void run() {
        awaitingTermination.setResource(false);
        while (!awaitingTermination.getResource()) {
            try {
                int bufferSize = datagramSocket.getReceiveBufferSize();
                byte[] buffer = new byte[bufferSize + 1];
                DatagramPacket packet = new DatagramPacket(buffer, bufferSize);
                datagramSocket.receive(packet);

                String requestContent = new String(packet.getData());

                if (requestContent.charAt(0) == 110) {
                    Thread.sleep(500);
                    continue;
                }

                AnnouncePackage announcePackage = serializerService.deserialize(requestContent, AnnouncePackage.class);

                if (null != announcePackage) {
                    InetAddress address = InetAddress.getByName(announcePackage.getIp());
                    addressContainer.setResource(address);
                }

            } catch (SocketException e) {
                log.log(this, "SocketException");
            } catch (IOException e) {
                log.log(this, "IOException: could not receive ");
            } catch (InterruptedException e) {
                log.log(this, "Interrupted exception when waiting for datagram.");
            }
        }
    }

    public void signalStop() {
        awaitingTermination.setResource(true);
    }

    public FinderService setListenPort(int listenPort) {
        try {
            datagramSocket = new DatagramSocket(listenPort);
        } catch (SocketException e) {
            log.log(this, "Socket Exception, could not listen.");
        }
        this.listenPort = listenPort;
        return this;
    }

    public FinderService setAddressContainer(AbstractSynchronizedResource<InetAddress> addressContainer) {
        this.addressContainer = addressContainer;
        return this;
    }
}
