package services;

import com.pi4j.io.i2c.I2CBus;
import com.pi4j.io.i2c.I2CDevice;
import com.pi4j.io.i2c.I2CFactory;
import entities.CompassDatum;

import java.io.IOException;
import java.util.zip.CRC32;

public class HeadingRetrievalService {
    protected static HeadingRetrievalService instance;
    private static final int LIS3MDL_ADDR = 0x1e;

    private static final int OUT_X_L = 0x28;
    private static final int OUT_X_H = 0x29;
    private static final int OUT_Y_L = 0x2a;
    private static final int OUT_Y_H = 0x2b;
    private static final int OUT_Z_L = 0x2c;
    private static final int OUT_Z_H = 0x2d;

    private static final int CTRL_REG1 = 0x020;
    private static final int CTRL_REG2 = 0x021;
    private static final int CTRL_REG3 = 0x022;
    private static final int CTRL_REG4 = 0x023;

    private static final int MD0 = 0x1;
    private static final int MD1 = 0x2;

    private I2CBus i2c;
    private I2CDevice device;

    public static HeadingRetrievalService getInstance() {
        if (null == instance) {
            instance = new HeadingRetrievalService();
        }

        return instance;
    }

    HeadingRetrievalService() {
        try {
            i2c = I2CFactory.getInstance(I2CBus.BUS_1);
            device = i2c.getDevice(LIS3MDL_ADDR);
            setContinuousUpdate();
        } catch (I2CFactory.UnsupportedBusNumberException e) {
            System.out.println("Unsupported Bus Number");
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("IOException");
            e.printStackTrace();
        }
    }

    public CompassDatum read() {
        int xl = 0, xh = 0, yl = 0, yh = 0, zl = 0, zh = 0;

        try {
            xl = device.read(OUT_X_L);
            xh = device.read(OUT_X_H);
            yl = device.read(OUT_Y_L);
            yh = device.read(OUT_Y_H);
            zl = device.read(OUT_Z_L);
            zh = device.read(OUT_Z_H);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return new CompassDatum()
                .setX(xl + xh << 8)
                .setY(yl + yh << 8)
                .setZ(zl + zh << 8);
    }

    public HeadingRetrievalService setContinuousUpdate() {
        try {
            byte register = (byte)device.read(CTRL_REG3);
            register &= 0xff ^ (MD0 & MD1);
            device.write(CTRL_REG3, register);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return this;
    }

    public HeadingRetrievalService setSingleUpdate() {
        try {
            byte register = (byte)device.read(CTRL_REG3);
            register &= 0xff ^ MD1;
            register |= MD0;
            device.write(CTRL_REG3, register);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return this;
    }

    public HeadingRetrievalService setIdle() {
        try {
            byte register = (byte)device.read(CTRL_REG3);
            register |= MD0;
            register |= MD1;
            device.write(CTRL_REG3, register);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return this;
    }
}
