package services;

import java.net.*;
import java.util.Enumeration;

public class IpAddressRetrievalService {
    private LogService log = LogService.getInstance();
    protected static IpAddressRetrievalService instance;

    public static IpAddressRetrievalService getInstance() {
        if (null == instance) {
            instance = new IpAddressRetrievalService();
        }

        return instance;
    }

    private IpAddressRetrievalService() {

    }

    public InetAddress getLocalIpAddress() {
        Enumeration<NetworkInterface> interfaces = null;
        try {
            interfaces = NetworkInterface.getNetworkInterfaces();
        } catch (SocketException e) {
            log.log(this, "Cannot retrieve network interfaces!");
        }

        if (null != interfaces)
        while (interfaces.hasMoreElements())
        {
            try {
                NetworkInterface networkInterface = interfaces.nextElement();
                if (networkInterface.isLoopback())
                    continue;    // Do not want to use the loopback interface.
                for (InterfaceAddress interfaceAddress : networkInterface.getInterfaceAddresses()) {
                    InetAddress broadcast = interfaceAddress.getAddress();

                    if (broadcast == null)
                        continue;
                    if (broadcast instanceof Inet6Address)
                        continue;

                    return broadcast;
                    // Do something with the address.
                }
            } catch (Exception e) {
                log.log(this, "Cannot retrieve broadcast address!");
            }
        }

        return null;
    }

    public InetAddress getBroadcastIpAddress() {
        Enumeration<NetworkInterface> interfaces = null;
        try {
             interfaces = NetworkInterface.getNetworkInterfaces();
        } catch (SocketException e) {
            log.log(this, "Cannot retrieve network interfaces!");
        }

        if (null != interfaces)
        while (interfaces.hasMoreElements())
        {
            try {
                NetworkInterface networkInterface = interfaces.nextElement();
                if (networkInterface.isLoopback())
                    continue;    // Do not want to use the loopback interface.
                for (InterfaceAddress interfaceAddress : networkInterface.getInterfaceAddresses()) {
                    InetAddress broadcast = interfaceAddress.getBroadcast();

                    if (broadcast == null)
                        continue;
                    if (broadcast instanceof Inet6Address)
                        continue;

                    return broadcast;
                    // Do something with the address.
                }
            } catch (Exception e) {
                log.log(this, "Cannot retrieve broadcast address!");
            }
        }

        return null;
    }
}
